package handler

import (
	"fmt"
	"net/http"
	"sirlab.de/go/knyt/user"
)

func (authMux *AuthMux) createCookie() *http.Cookie {
	return &http.Cookie{
		Name:     "knyt-auth",
		Path:     "/",
		HttpOnly: true,
		MaxAge:   -1,
		SameSite: http.SameSiteLaxMode,
	}
}

func (authMux *AuthMux) Logout(w http.ResponseWriter, r *http.Request) {
	http.SetCookie(w, authMux.createCookie())
	w.Header().Add("Content-Type", "text/plain")
	fmt.Fprintf(w, "ok")
}

func (authMux *AuthMux) Login(w http.ResponseWriter, r *http.Request) {
	usr, err := authMux.checkCode(r)
	if err != nil {
		http.SetCookie(w, authMux.createCookie())
		authMux.accessDenied(w, r)
		return
	}

	cookie := authMux.createCookie()
	cookie.Value = usr.GetId() + ":" + usr.GetAuthCode()
	cookie.MaxAge = 0
	http.SetCookie(w, cookie)
	w.Header().Add("Content-Type", "text/plain")
	fmt.Fprintf(w, "ok")
}

func (authMux *AuthMux) checkCode(r *http.Request) (*user.User, error) {
	r.ParseForm()
	form := r.Form
	code := form.Get("code")

	if len(code) != 6 {
		return nil, fmt.Errorf("invalid code")
	}

	usr, err := authMux.app.GetUserByAuthcode(code)
	if err != nil {
		return nil, fmt.Errorf("invalid code")
	}

	return usr, nil
}
